package cn.com.poc.knowledge.rest.impl;

import cn.com.poc.common.utils.Assert;
import cn.com.poc.common.utils.BlContext;
import cn.com.poc.common.utils.DatabaseUtil;
import cn.com.poc.common.utils.StringUtils;
import cn.com.poc.knowledge.convert.BizKnowledgeDatabaseConvert;
import cn.com.poc.knowledge.dto.BizKnowledgeDatabaseDto;
import cn.com.poc.knowledge.dto.DatabaseTestContentDto;
import cn.com.poc.knowledge.dto.KnowledgeDatabaseSearchDto;
import cn.com.poc.knowledge.entity.BizKnowledgeDatabaseEntity;
import cn.com.poc.knowledge.query.KnowledgeDatabaseQueryInfoQueryCondition;
import cn.com.poc.knowledge.query.KnowledgeDatabaseQueryInfoQueryItem;
import cn.com.poc.knowledge.rest.DatabaseRest;
import cn.com.poc.knowledge.service.BizKnowledgeDatabaseService;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.data.model.PagingInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class DatabaseRestImpl implements DatabaseRest {


    @Resource
    private BizKnowledgeDatabaseService bizKnowledgeDatabaseService;

    @Override
    public BizKnowledgeDatabaseDto create(BizKnowledgeDatabaseDto dto) {
        Assert.notNull(dto, "response can not be null");
        Assert.notBlank(dto.getTitle(), "title can not be blank");
        Assert.notBlank(dto.getDbHost(), "Database Host must not be blank");
        Assert.notBlank(dto.getDbType(), "Database Type must not be blank");
        Assert.notNull(dto.getDbPort(), "Database Port must not be blank");
        Assert.notBlank(dto.getDbName(), "Database Name must not be blank");
        Assert.notBlank(dto.getDbUsername(), "Database Username must not be blank");
        Assert.notBlank(dto.getDbPassword(), "Database Password must not be blank");

        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        Long userId = userBaseEntity.getUserId();
        BizKnowledgeDatabaseEntity bizKnowledgeDatabaseEntity = BizKnowledgeDatabaseConvert.dtoToEntity(dto);
        bizKnowledgeDatabaseEntity.setMemberId(userId.intValue());
        BizKnowledgeDatabaseEntity saveEntity = bizKnowledgeDatabaseService.save(bizKnowledgeDatabaseEntity);
        if (saveEntity == null) {
            throw new BusinessException("create database error");
        }

        return BizKnowledgeDatabaseConvert.entityToDto(saveEntity);
    }

    @Override
    public DatabaseTestContentDto testConnection(BizKnowledgeDatabaseDto dto) {
        Assert.notBlank(dto.getDbHost(), "Database Host must not be blank");
        Assert.notNull(dto.getDbPort(), "Database Port must not be blank");
        Assert.notBlank(dto.getDbName(), "Database Name must not be blank");
        Assert.notBlank(dto.getDbUsername(), "Database Username must not be blank");
        Assert.notBlank(dto.getDbPassword(), "Database Password must not be blank");
        boolean connect = DatabaseUtil.testConnect(dto.getDbHost(), dto.getDbPort(), dto.getDbUsername(), dto.getDbPassword(), dto.getDbName());
        DatabaseTestContentDto result = new DatabaseTestContentDto();
        result.setStatus(connect ? "success" : "fail");
        return result;
    }

    @Override
    public BizKnowledgeDatabaseDto get(Integer id) {
        Assert.notNull(id, "id can not be null");
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        Long userId = userBaseEntity.getUserId();

        BizKnowledgeDatabaseEntity bizKnowledgeDatabaseEntity = bizKnowledgeDatabaseService.get(id.longValue());
        if (bizKnowledgeDatabaseEntity == null && !bizKnowledgeDatabaseEntity.getMemberId().equals(userId.intValue())) {
            throw new BusinessException("no database found");
        }
        return BizKnowledgeDatabaseConvert.entityToDto(bizKnowledgeDatabaseEntity);
    }

    @Override
    public List<BizKnowledgeDatabaseDto> getList(KnowledgeDatabaseSearchDto knowledgeDatabaseSearchDto, PagingInfo pagingInfo) {
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        Long userId = userBaseEntity.getUserId();
        KnowledgeDatabaseQueryInfoQueryCondition condition = new KnowledgeDatabaseQueryInfoQueryCondition();
        condition.setMemberId(userId.intValue());
        if (StringUtils.isNotBlank(knowledgeDatabaseSearchDto.getSearch())) {
            condition.setSearch(knowledgeDatabaseSearchDto.getSearch());
        }
        List<KnowledgeDatabaseQueryInfoQueryItem> items = bizKnowledgeDatabaseService.queryKnowledgeDatabaseQueryInfo(condition, pagingInfo);
        List<BizKnowledgeDatabaseDto> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(items)) {
            result = items.stream().map(BizKnowledgeDatabaseConvert::itemToDto).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public boolean delete(Integer id) {
        Assert.notNull(id, "id can not be null");
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        Long userId = userBaseEntity.getUserId();
        BizKnowledgeDatabaseEntity entity = bizKnowledgeDatabaseService.get(id.longValue());
        if (entity == null) {
            return false;
        }
        if (!entity.getMemberId().equals(userId.intValue())) {
            throw new BusinessException("no permission");
        }
        bizKnowledgeDatabaseService.deletedById(id.longValue());
        return true;
    }

    @Override
    public BizKnowledgeDatabaseDto update(BizKnowledgeDatabaseDto dto) {
        Assert.notNull(dto, "response can not be null");
        Assert.notNull(dto.getId(), "id can not be null");
        BizKnowledgeDatabaseEntity bizKnowledgeDatabaseEntity = BizKnowledgeDatabaseConvert.dtoToEntity(dto);
        BizKnowledgeDatabaseEntity updateEntity = bizKnowledgeDatabaseService.update(bizKnowledgeDatabaseEntity);
        if (updateEntity == null) {
            throw new BusinessException("update database info error");
        }
        return BizKnowledgeDatabaseConvert.entityToDto(updateEntity);
    }
}